unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Clipbrd, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls, Menus, HSButton;

const
  //Password mask consts
  TLowerAlpha = 'abcdefghijklmnopqrstuvwxyz';
  TUpperAlpha = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
  TNumber     = '0123456789';
  TSpecial    = '`!"?$?%^&*()''_-+={[}]:;@~#|\<,>./';

type

  { Tfrmmain }

  Tfrmmain = class(TForm)
    CheckGroup1: TCheckGroup;
    chkLower: TCheckBox;
    chkNumber: TCheckBox;
    chkSpecial: TCheckBox;
    chkUpper: TCheckBox;
    HSBtnMemory: THSButton;
    HSBtnGenerate: THSButton;
    HSBtnSave: THSButton;
    HSBtnAuthor: THSButton;
    Image1: TImage;
    LstPws: TListView;
    lblNumber: TLabel;
    MenuItem1: TMenuItem;
    PopupMenu1: TPopupMenu;
    SFD: TSaveDialog;
    txtLength: TEdit;
    lblLength: TLabel;
    txtNum: TEdit;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure HSBtnAuthorClick(Sender: TObject);
    procedure HSBtnMemoryClick(Sender: TObject);
    procedure HSBtnGenerateClick(Sender: TObject);
    procedure HSBtnSaveClick(Sender: TObject);
    procedure LstPwsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  frmmain: Tfrmmain;
  rChar: integer;
  SelectedPW : String ;

function isNumber(buff: string): boolean;
function GetPassword(PasswordMask: string; pLength: integer): string;

implementation

{$R *.lfm}

{ Tfrmmain }

function ListToString(LV: TListView): string;
var
  counter: integer;
  buff: string;
begin
  //Init var
  buff := '';
  //Loop tho the items in listview.
  for counter := 0 to LV.Items.Count - 1 do
  begin
    buff := buff + LV.Items[counter].Caption + #13 + #10;
  end;
  //Return result
  Result := buff;
end;

function isNumber(buff: string): boolean;
begin
  //Check for empty string
  if buff = '' then
  begin
    Result := False;
    exit;
  end;
  try
    StrToInt(buff);
    Result := True;
  except
    Result := False;
  end;
end;

function GetPassword(PasswordMask: string; pLength: integer): string;
var
  Counter: integer;
  rPass: string;
begin

  rPass := '';

  for Counter := 1 to pLength do
  begin
    //Get randdom char from password set
    rChar := Random(Length(PasswordMask));
    //Build random password
    rPass := rPass + Copy(PasswordMask, rChar, 1);
  end;
  //Return password
  Result := rPass;
end;

procedure Tfrmmain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  Clipboard.AsText := '';
end;

procedure Tfrmmain.HSBtnMemoryClick(Sender: TObject);
begin
  If SelectedPW > '' then
  begin
    Clipboard.AsText := SelectedPW;
  end;

end;

procedure Tfrmmain.HSBtnGenerateClick(Sender: TObject);
var
  pMask: string;
  Counter: integer;
  pLen: integer;
  pNum: integer;
  LItem: TListItem;

begin
  //Set defaults
  pMask := '';
  LstPws.Items.Clear;
  LstPws.Column[0].Caption := '0 passwords generated';
  HSBtnSave.Enabled:= True;

  //Check for vaild password length
  if not isNumber(txtLength.Text) then
  begin
    MessageDlg('Length is not a vaild integer.',
      mtWarning, [mbOK], 0);
    exit;
  end;
  //Check for vaid number of passwords
  if not isNumber(txtNum.Text) then
  begin
    MessageDlg('Number of passwords is not a vaild integer.',
      mtWarning, [mbOK], 0);
    exit;
  end;

  //Assign the variables
  pLen := StrToInt(txtLength.Text);
  pNum := StrToInt(txtNum.Text);

  //Check for password length
  if pLen <= 0 then
  begin
    MessageDlg('Your password length needs to be greater then zero.',
      mtWarning, [mbOK], 0);
    exit;
  end;

  //Check number of passwords
  if pNum <= 0 then
  begin
    MessageDlg('The number of passwords needs to be greater than zero.',
      mtWarning, [mbOK], 0);
    exit;
  end;

  //Create password mask.
  if chkUpper.Checked then
  begin
    //Create upper alpha random password
    pMask := TUpperAlpha;
  end;

  if chkLower.Checked then
  begin
    //Create lowercase alpha random password
    pMask := pMask + TLowerAlpha;
  end;

  if chkNumber.Checked then
  begin
    //Create number random password
    pMask := pMask + TNumber;
  end;

  if chkSpecial.Checked then
  begin
    //Create special sybmols random password
    pMask := pMask + TSpecial;
  end;

  //Check password mask length
  if Length(pMask) = 0 then
  begin
    MessageDlg('Please select a password option.', mtInformation, [mbOK], 0);
    exit;
  end;
  //Set listview header text
  LstPws.Column[0].Caption := IntToStr(pNum) + ' '+'passwords generated';

  //Randomize
  Randomize;
  for Counter := 1 to pNum do
  begin
    //Add item to listview control
    LItem := LstPws.Items.Add;
    //Add random password as item
    LItem.Caption := GetPassword(pMask, pLen);
  end;
  //
end;

procedure Tfrmmain.HSBtnSaveClick(Sender: TObject);
var
  T: TextFile;
begin
  // Set the name of the file that will be created
  AssignFile(T, SFD.FileName);
  LstPws.Column[0].Caption := '';

  // Use exceptions to catch errors (this is the default so not absolutely requried)
  {$I+}

  // Embed the file creation in a try/except block to handle errors gracefully
  try
    // Create the file, write some text and close it.
    // Check if there is anything to be saved.
     if LstPws.Items.Count = 0
     then
       begin
       MessageDlg('Nothing to save.', mtInformation, [mbOK], 0);
       exit;
       end
     else
       begin
         //Setup save dialog props
         SFD.Title := 'Save Passwords';
         SFD.Filter := 'text|*.Txt';


         if SFD.Execute
         then
           begin
             //Save the list of passwords
             AssignFile(T, SFD.FileName);
             Rewrite(T);
             //Write the listview contents to the filename
             Write(T, ListToString(LstPws));
             CloseFile(T);
             HSBtnSave.Enabled:= False;
           end
         else


           exit;
         end;
  except
    // If there was an error the reason can be found here
    on E: EInOutError do
      writeln('File handling error occurred. Details: ', E.ClassName, '/', E.Message);
  end;

  Clipboard.AsText := '';
  LstPws.Clear;
  LstPws.Refresh;

  HSBtnMemory.Enabled := False;


end;
procedure Tfrmmain.HSBtnAuthorClick(Sender: TObject);
var
  msg: string;
begin
  //Text for messagebox
  msg := self.Caption + ' v2.0' + #13 + #10 + ' originally developed by Ben Jones'
                      + #13 + #10 + 'and edited by Blaise Pascal Magazine September 2021';
  //Show our Messagebox
  MessageDlg(msg, mtInformation, [mbOK], 0);

end;

procedure Tfrmmain.LstPwsSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
Var SelectedPW : String ;
begin

  If Selected then
  begin
    SelectedPW := Item.Caption ;
    HSBtnMemory.Enabled:= True;
    HSBtnSave.Enabled:= True;
    Clipboard.AsText := SelectedPW;
  end;
end;




end.

